﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Windows.Foundation.Collections;

namespace VSLiveWinRTData
{
    /// <summary>
    /// Class that works for data bindings for WinRT list based controls
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class ExtendedObservableCollection<T> : ObservableCollection<T>, IObservableVector<object>
    {

        public ExtendedObservableCollection(IEnumerable<T> collection)
            : base(collection)
        {
        }

        /// <summary>
        /// Raises collection changed event
        /// </summary>
        /// <param name="e">Event arguments for collection changed event</param>
        protected override void OnCollectionChanged(System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            base.OnCollectionChanged(e);
            switch (e.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    OnVectorChanged(CollectionChange.ItemInserted, (uint)e.NewStartingIndex);
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Move:
                    OnVectorChanged(CollectionChange.Reset, (uint)e.NewStartingIndex);
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                    OnVectorChanged(CollectionChange.ItemRemoved, (uint)e.OldStartingIndex);
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Replace:
                    OnVectorChanged(CollectionChange.ItemChanged, (uint)e.NewStartingIndex);
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Reset:
                    OnVectorChanged(CollectionChange.Reset, (uint)e.NewStartingIndex);
                    break;
                default:
                    break;
            }
        }
        /// <summary>
        /// Raises VectorChanged event to signal to UI that collection has changed
        /// </summary>
        /// <param name="collectionChange">Type of change</param>
        /// <param name="index">Index of change</param>
        protected void OnVectorChanged(CollectionChange collectionChange, uint index)
        {
            if (VectorChanged != null)
            {
                VectorChanged(this, new VectorChangedEventArgs(collectionChange, index));
            }
        }

        /// <summary>
        /// Public event that is raised when collection changes
        /// </summary>
        public event VectorChangedEventHandler<object> VectorChanged;

        /// <summary>
        /// Index of an item in the collection
        /// </summary>
        /// <param name="item">Item to get index of</param>
        /// <returns>Index of an item in the collection</returns>
        public int IndexOf(object item)
        {
            return base.IndexOf((T)item);
        }

        /// <summary>
        /// Insert an item into the collection
        /// </summary>
        /// <param name="index">Index to insert item at</param>
        /// <param name="item">item to insert</param>
        public void Insert(int index, object item)
        {
            base.InsertItem(index, (T)item);
        }

        /// <summary>
        /// Get an item at an index
        /// </summary>
        /// <param name="index">Index for an item</param>
        /// <returns>Item at the specified index</returns>
        public new object this[int index]
        {
            get
            {
                return base[index];
            }
            set
            {
                base[index] = (T)value;
            }
        }

        /// <summary>
        /// Add an item to the collection
        /// </summary>
        /// <param name="item">item to add</param>
        public void Add(object item)
        {
            base.Add((T)item);
        }

        /// <summary>
        /// Determine if an item is part of the collection
        /// </summary>
        /// <param name="item">item to check</param>
        /// <returns>True if item belongs to the collection</returns>
        public bool Contains(object item)
        {
            return base.Contains((T)item);
        }

        /// <summary>
        /// Copy items to an array
        /// </summary>
        /// <param name="array">Array of items to copy</param>
        /// <param name="arrayIndex">Index position to copy through</param>
        public void CopyTo(object[] array, int arrayIndex)
        {
            T[] newArray = new T[array.Length];
            for (int i = 0; i < array.Length; i++)
            {
                newArray[i] = (T)array[i];
            }
            CopyTo(newArray, arrayIndex);
        }

        /// <summary>
        /// Is collection read only
        /// </summary>
        public bool IsReadOnly
        {
            get { return false; }
        }

        /// <summary>
        /// Remove an item from collection
        /// </summary>
        /// <param name="item">item to remove</param>
        /// <returns>True if items is removed, false if it was not part of the collection</returns>
        public bool Remove(object item)
        {
            if (Contains(item))
            {
                base.Remove((T)item);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Get enumerator for items in the collection
        /// </summary>
        /// <returns>IEnumerator object</returns>
        public new IEnumerator<object> GetEnumerator()
        {
            return base.GetEnumerator() as IEnumerator<object>;
        }
    }
}
